% function to calibrate

function [kappa, a_thresh, eta1, delta_s, k_ss, ss] = calibrate_unified(P, shock, target, init_guess)

% Goal: pick eta1, a_thresh, k_ss so that:
% (a) steady state interest rate is 0.06
% (b) debt is 0.45
% (c) MPC out of capital gains is 2.7

options = optimoptions('fsolve', 'Display', 'off', 'FunctionTolerance', 1e-7, 'OptimalityTolerance', 1e-7, 'StepTolerance', 1e-7);
options_fzero = optimset('Display', 'off', 'TolX', 1e-4, 'TolFun', 1e-4);

d_target = target(1);
MPC_target = target(2);
state_level_target = target(3);
r_target = target(4);

kappa_guess = init_guess(1);
a_thresh_guess = init_guess(2);
eta1_guess = init_guess(3);
delta_s_guess = init_guess(4);
k_guess = init_guess(5);

%%% 1. solve for k  

jacobian_fn = @(k) P.jacobian(k, 1, 1, P.alpha,P.gamma(0),P.land_share,P.sigma,k);
MPK_F = @(k) table(jacobian_fn(k)).Var1(1) - (r_target + P.delta);
[k , fval] = fsolve(MPK_F, k_guess, options);

Yhat = P.f(k,1,1,P.alpha,P.gamma(0),P.land_share,P.sigma,k);


k_ss = k;
% update a_ss
jacobian_k = jacobian_fn(k);
w_s = jacobian_k(2);
w_b = jacobian_k(3);
y_s = w_s; y_b = w_b;
omega_s = y_s/(y_s+y_b);  

pi = Yhat - y_s - y_b;
I = P.delta*k;
J = (pi - I)/r_target;

P.B0 = P.B(0);  
Tau = P.g(0) + r_target*P.B0 - P.tau_s(0)*y_s - P.tau_b(0)*y_b;

h_s = ((1-P.tau_s(0)) * y_s - Tau*omega_s) / r_target;
a_ss = h_s + d_target + P.B0 + P.kappa_s * J;
shock.omega_s = @(t) omega_s; % CHECK
P.omega_s = omega_s;


%%% define function for delta_s & guess
F2 = @(delta_s) target_state(delta_s, a_ss, P, shock, r_target, MPC_target, state_level_target, a_thresh_guess, eta1_guess);

% solve
solve_for_delta_s = 0; % triggered only for state-level target
if abs(F2(delta_s_guess)) < 1e-4 || ~solve_for_delta_s  
    x = delta_s_guess; 
else
    [x, fval] = fzero(F2, [1e-3,0.5], options_fzero);
end

[~, a_thresh, eta1] = F2(x);
delta_s = x;


%%% solve for kappa
p = Yhat / r_target; 
F3 = @(x)x * p - P.kappa0 - d_target;
[kappa, fval] = fzero(F3, kappa_guess, options_fzero);


%%% Compute updated ss
P.delta_s = delta_s; P.delta_b = delta_s;
P.kappa = kappa;
P.a_thresh = a_thresh; P.eta1 = eta1; P.kss = k_ss;

if delta_s < 0
    warning('delta_s negative!');  
end

[~, ss] = steadystate_unified(P);

end






function [target_state, a_thresh, eta1] = target_state(delta_s, a_ss, P, shock, r_target, MPC_target, state_level_target, a_thresh_guess, eta1_guess)

% %%% 1. aux functions
eta0 = P.eta0;

%%% 2. condition for steady state interest rate:
target1 = @(x) r_target - P.rho_s * (1 + delta_s / P.rho_s) /...
          (1 + delta_s / P.rho_s * P.eta_s(a_ss, x(1), eta0, x(2)));

%%% 3. MPC capital gains:
eps_eta = @(a_thresh, eta1) P.eta_s_prime(a_ss, a_thresh, eta0, eta1) * a_ss / P.eta_s(a_ss, a_thresh, eta0, eta1);

% MPC out of K gains
ratio = r_target / (P.rho_s + delta_s);
MPC_capgain_theory = @(a_thresh, eta1) r_target * (1 - 1 / (2 * ratio) * ...
                     (1 - sqrt(1 - 4 * (1 - ratio) * ratio * eps_eta(a_thresh, eta1))));

target2 = @(x) MPC_target - MPC_capgain_theory(x(1), x(2));


%%% 4. solve for a_thresh and eta1
x0 = [a_thresh_guess, eta1_guess];
options = optimoptions('fsolve', 'Display', 'off', 'FunctionTolerance', 1e-20, 'OptimalityTolerance', 1e-12, 'StepTolerance', 1e-12);
F = @(x) [target1(x); target2(x)];
[x, fval] = fsolve(F, x0, options);

a_thresh = x(1);
eta1 = x(2);

P.a_thresh = a_thresh;
P.eta1 = eta1;


%%% 5. given a_thresh & eta & delta_s solve state level
shock.r_exog = @(t) r_target; 
T = 150;

% feed in inequality shock
Delta = 0.04;  % after taxes!
time = 25;

% linear
P.d0 = a_ss;
P.tau_s = 0; P.tau_b = 0; P.B0 = 0; P.Tau = @(B) B-P.B0; P.g = 0;
shock.omega_s = @(t) Delta*(t<time).*t/time + Delta*(t>=time);
shock.omega_s_dot = @(t) Delta*(t<time);
shock.r_exog = @(t) r_target;

td = compute_pe_unified(P,T,shock,0);

% compute saving increase per inequality increase
time_period = time;  % first X years

% function to evaluate path at a given time
eval_path = @(field, t) interp1(td.t,td.(field),t);

total_inc_share_increase = eval_path('total_inc_s',time_period) - eval_path('total_inc_s',0);
sav_increase = eval_path('d',time_period) - eval_path('d',0);
sav_to_ineq = sav_increase / total_inc_share_increase;  % ratio

target_state = sav_to_ineq - state_level_target;

disp(['Current state level coefficient:' num2str(sav_to_ineq)])

end