% This file computes PE for the model

function td = compute_pe_unified(P,T,shock,previousrun)

% aux
reltol = read_opt(P,'reltol',1e-6);
abstol = read_opt(P,'abstol',1e-6);
NMax   = read_opt(P,'NMax',5000);
options = bvpset('RelTol',reltol,'AbsTol',abstol,'NMax',NMax);


%%% 1. Set initial guess
% Inputs we need:
%  - shock.omega_s [only increase is important; should be net of taxes]
%  - shock.r_exog
%  - P.d0 [includes all assets that don't change]
P.r_ss = shock.r_exog(1e6);
h_s = shock.omega_s(1e6) / P.r_ss;
c_s = P.r_ss * (P.d0 + h_s);
y_guess =  [c_s P.d0 h_s 0]';
P.n_y = length(y_guess);
P.T = T;

% Set initial time grid
t_guess = linspace(0,T,1000); 

test_initial = odefun(0, y_guess, P, shock);
test_final = bcfun(y_guess, y_guess, P, shock);

% 2. Set entire path for guess
if exist('previousrun','var')
    if isstruct(previousrun)
        % use guess from previous run
        t_guess = previousrun.t;
        y_guess = previousrun.y_all(:,end)';
    end
end

% 3. Set the functions used for bvp4c
myodefun = @(t,y) odefun(t,y,P,shock);
mybcfun  = @(y0,yT) bcfun(y0,yT,P,shock);

solinit = bvpinit(t_guess,y_guess); 

tic;
sol = bvp4c(myodefun,...
            mybcfun,...
            solinit,...
            options);
toc;

td.y_all = sol.y;
td.t = sol.x; i=1;
td.c_s = sol.y(i,:); i=i+1;
td.d = sol.y(i,:); i=i+1;
td.h_s = sol.y(i,:); i=i+1;
td.cap_r = sol.y(i,:);
td.omega_s = shock.omega_s(td.t);
td.r = shock.r_exog(td.t);
td.total_inc_s = td.omega_s + td.r .* td.d;

td.a_s = td.h_s + td.d;
td.wealth_income_s = td.a_s ./ td.total_inc_s;

td.net_worth_s = td.a_s;

end



%%% ODEFUN %%%
function [ydot, extra] = odefun(t,y,P,shock)

% this function computes the derivative of the ode system
i=1;
c_s = y(i); i = i + 1;
d = y(i); i = i + 1;
h_s = y(i); 

% Read out paths
r = shock.r_exog(t);
omega_s = shock.omega_s(t); 

% Aux objects
a_s = h_s + d; % total wealth


% Budget Constraint
d_dot = r * d + omega_s - c_s;

% Euler Equation
c_s_dot = c_s * 1/P.inverse_eis * (r - P.rho_s - P.delta_s + P.delta_s * (c_s / P.rho_s / a_s)^P.inverse_eis * P.eta_s(a_s,P.a_thresh,P.eta0,P.eta1));

% Human capital
h_s_dot = r * h_s - omega_s;

% Cumulative interest rate
cap_r_dot = r - P.r_ss;

% Put everything together
ydot = [c_s_dot, d_dot, h_s_dot, cap_r_dot]';

if any(isnan(ydot))
    warning('NaN encountered in odefun')
end

% extra output
extra.r = r; 
extra.omega_s = omega_s; 
extra.d_dot = d_dot;

extra.check = c_s + d_dot - r*d - omega_s;
if abs(extra.check) > 1e-1
    1;
end

end


%%% BCFUN %%%
function res = bcfun(y_0,y_inf,P,shock)
% this function imposes the boundary conditions

i = 1;
c_s_inf = y_inf(i);
i = i + 1;
d0 = y_0(i); dinf = y_inf(i); i = i + 1;
h_s_inf = y_inf(i); i = i + 1;
cap_r_0 = y_0(i,1); 

a_s_inf = h_s_inf + dinf;
r_inf = shock.r_exog(1e6);
omega_s_inf = shock.omega_s(1e6);

% boundary conditions at t=0 or t=T
k = 1; res = zeros(P.n_y,1);
res(k) = d0 - P.d0; k = k+1;  % initial debt
res(k) = h_s_inf - omega_s_inf / r_inf; k = k + 1; % human wealth at T
res(k) = r_inf - P.rho_s - P.delta_s + P.delta_s * c_s_inf / P.rho_s / a_s_inf * P.eta_s(a_s_inf,P.a_thresh,P.eta0,P.eta1); k = k + 1; % c_dot = 0
res(k) = cap_r_0;  % initial R

if any(isnan(res))
    warning('NaN encountered in bcfun')
end

end







