% compute ss

function [ss_vec, ss] = steadystate_unified(P) 

% I. solve with nat rate
r_lb = 1e-8; r_ub = 2; n = 1;
fun = @(r) endog_nr(n, r, P);
if sign(fun(r_lb)) == sign(fun(r_ub))
    1;
end

r = fzero(fun, [r_lb, r_ub]);
[ss, ~] = ss_equations(n, r, P);

options = optimoptions('fsolve','FunctionTolerance',1e-13,...
    'StepTolerance', 1e-13, 'OptimalityTolerance', 1e-13, 'Display','off', 'MaxFunctionEvaluations', 2000);


% II. Check if nat rate > ZLB and T_r_exog < inf
r_ZLB = P.r_ZLB; 
if r < r_ZLB || P.T_exog_to_endog == inf || P.T_endog_to_exog < inf
    % Solve model with exog r and endog Yhat
    if r < r_ZLB
        r = r_ZLB;
    else
        r = P.r_exog; 
    end
    n = fsolve(@(n) endog_nr(n, r, P), 1, options);
    [ss, ~] = ss_equations(n, r, P);
end

% assemble as vector
ss_vec = [ss.c_s, ss.c_b, ss.d, ss.h_s, ss.h_b, ss.n, P.kappa, ss.k, ss.J, ss.Q, ss.q, ss.p]';
assert(ss.Yhat - ss.c_s - ss.c_b - ss.I - P.g(0) < 1e-5)
if any(isnan(ss_vec))
    warning('NaN encountered in steady state')
end

end

function cond = endog_nr(n, r, P)
[~, cond] = ss_equations(n, r, P);
end


function [ss, cond] = ss_equations(n, r, P)
k_ub = 500;
k_lb = 1e-8;

jacobian_fn = @(k)P.jacobian(k, n, n, P.alpha,P.gamma(0),P.land_share,P.sigma,P.kss); %P.kss
MPK_F = @(k) table(jacobian_fn(k)).Var1(1) - (r + P.delta);

k = fzero(MPK_F, [k_lb, k_ub]);
Yhat = P.f(k,n,n,P.alpha,P.gamma(0),P.land_share,P.sigma,P.kss); 

% firm and household
jacobian = jacobian_fn(k);
w_s = jacobian(2);
w_b = jacobian(3);
y_s = w_s*n; y_b = w_b*n;
omega_s = y_s/(y_s+y_b); omega_b = y_b/(y_s+y_b);
tax_share_s = P.chi * omega_s + P.phi(0);
tax_share_b = 1 - P.chi * omega_s - P.phi(0);
pi = Yhat - y_s - y_b;
I = P.delta*k;
Q = 1;
J = (pi - I)/r;
div = r*J;
q = 1/(r + P.nu);
Y_pot = P.f(k,1,1,P.alpha,P.gamma(0),P.land_share,P.sigma,P.kss);

% price of land
p = Yhat / r;

if P.exog_debt == 1
    d = P.exog_debt_d;
else
    d = P.kappa * p - P.kappa0;  % Constraint enters here!  P.kappa * h_b - P.kappa0;
end

% calculate Tau from government BC
% government 
B = P.B(0);
g = P.g(0);
fun = @(Tau) P.tau_s(0)*y_s + P.tau_b(0)*y_b + Tau - g - r*B + ...
             P.xi*(((1-P.tau_s(0))*y_s-Tau*omega_s)/r+d+B+J);
Tau = fzero(fun, 0);

h_s = ((1-P.tau_s(0)) * y_s - Tau*tax_share_s) / r;
h_b = ((1-P.tau_b(0)) * y_b - Tau*tax_share_b) / r;

% household 
a_s = h_s + d + B + P.kappa_s * J;
a_b = h_b - d + P.kappa_b * J;

c_s = r * a_s - P.xi*a_s;
c_b = r * a_b;

ss.c_s = c_s; ss.c_b = c_b; ss.d = d; ss.h_s = h_s; ss.h_b = h_b;
ss.r = r; ss.Tau = Tau; ss.a_s = a_s; ss.B = B; ss.Yhat = Yhat;
ss.a_b = a_b; 
ss.k = k; ss.n = n; ss.I = I; ss.J = J; ss.Q = Q; ss.div = div;
ss.w_s = w_s; ss.w_b = w_b;
ss.omega_s = omega_s; ss.omega_b = omega_b;
ss.q = q; ss.p = p;

% Conditions
cond = r - P.rho_s - P.delta_s + P.delta_s * (c_s / (P.rho_s * a_s))^P.inverse_eis * P.eta_s(a_s/Y_pot, P.a_thresh, P.eta0, P.eta1)...
        - P.delta_s/P.rho_s * P.xi * P.eta_s(a_s/Y_pot, P.a_thresh, P.eta0, P.eta1);
end